package dev.adriankuta.pixabay.data.paging

import androidx.paging.PagingSource
import androidx.paging.PagingState
import dev.adriankuta.pixabay.data.model.PixabayImage
import dev.adriankuta.pixabay.data.network.PixabayService
import dev.adriankuta.pixabay.data.repository.PixabayImageRepository.Companion.NETWORK_PAGE_SIZE
import retrofit2.HttpException
import java.io.IOException

private const val STARTING_PAGE_INDEX = 1

internal class PixabayPagingSource(
    private val pixabayService: PixabayService,
    private val query: String
) : PagingSource<Int, PixabayImage>() {

    override suspend fun load(params: LoadParams<Int>): LoadResult<Int, PixabayImage> {
        val position = params.key ?: STARTING_PAGE_INDEX
        return try {
            val response = pixabayService.searchImages(query, position, params.loadSize)
            val results = response.hits.map { PixabayImage(it) }
            val nextKey = if (results.isEmpty()) {
                null
            } else {
                // initial load size = 3 * NETWORK_PAGE_SIZE
                // ensure we're not requesting duplicating items, at the 2nd request
                position + (params.loadSize / NETWORK_PAGE_SIZE)
            }
            LoadResult.Page(
                data = results,
                prevKey = if (position == STARTING_PAGE_INDEX) null else position - 1,
                nextKey = nextKey
            )
        } catch (exception: IOException) {
            return LoadResult.Error(exception)
        } catch (exception: HttpException) {
            return LoadResult.Error(exception)
        }
    }

    // The refresh key is used for subsequent refresh calls to PagingSource.load after the initial load
    override fun getRefreshKey(state: PagingState<Int, PixabayImage>): Int? {
        // We need to get the previous key (or next key if previous is null) of the page
        // that was closest to the most recently accessed index.
        // Anchor position is the most recently accessed index
        return state.anchorPosition?.let { anchorPosition ->
            state.closestPageToPosition(anchorPosition)?.prevKey?.plus(1)
                ?: state.closestPageToPosition(anchorPosition)?.nextKey?.minus(1)
        }
    }
}