@file:OptIn(ExperimentalPagingApi::class)

package dev.adriankuta.pixabay.data.paging

import androidx.paging.ExperimentalPagingApi
import androidx.paging.LoadType
import androidx.paging.PagingState
import androidx.paging.RemoteMediator
import androidx.room.withTransaction
import dev.adriankuta.pixabay.data.model.PixabayImage
import dev.adriankuta.pixabay.data.network.PixabayService
import dev.adriankuta.pixabay.data.room.AppDatabase
import dev.adriankuta.pixabay.data.room.entity.PixabayImageEntity
import dev.adriankuta.pixabay.data.room.entity.RemoteKeys
import retrofit2.HttpException
import java.io.IOException

private const val STARTING_PAGE_INDEX = 1

internal class PixabayRemoteMediator(
    private val query: String,
    private val database: AppDatabase,
    private val pixabayService: PixabayService
) : RemoteMediator<Int, PixabayImageEntity>() {

    private val imagesDao = database.imagesDao()
    private val remoteKeysDao = database.remoteKeysDao()


    override suspend fun load(
        loadType: LoadType,
        state: PagingState<Int, PixabayImageEntity>
    ): MediatorResult {
        val page = when (loadType) {
            LoadType.REFRESH -> {
                val remoteKeys = getRemoteKeyClosestToCurrentPosition(state)
                remoteKeys?.nextKey?.minus(1) ?: STARTING_PAGE_INDEX
            }

            LoadType.PREPEND -> {
                val remoteKeys = getRemoteKeyForFirstItem(state)
                // If remoteKeys is null, that means the refresh result is not in the database yet.
                val prevKey = remoteKeys?.prevKey
                    ?: return MediatorResult.Success(endOfPaginationReached = remoteKeys != null)
                prevKey
            }

            LoadType.APPEND -> {
                val remoteKeys = getRemoteKeyForLastItem(state)
                // If remoteKeys is null, that means the refresh result is not in the database yet.
                // We can return Success with endOfPaginationReached = false because Paging
                // will call this method again if RemoteKeys becomes non-null.
                // If remoteKeys is NOT NULL but its nextKey is null, that means we've reached
                // the end of pagination for append.
                val nextKey = remoteKeys?.nextKey
                    ?: return MediatorResult.Success(endOfPaginationReached = remoteKeys != null)
                nextKey
            }
        }

        try {
            val response = pixabayService.searchImages(query, page, state.config.pageSize)
            val imageEntities = response.hits.map { PixabayImageEntity(it, query) }
            val endOfPaginationReached = response.hits.isEmpty()
            database.withTransaction {
                if (loadType == LoadType.REFRESH) {
                    imagesDao.clearAll()
                    remoteKeysDao.clearRemoteKeys()
                }
                val prevKey = if (page == STARTING_PAGE_INDEX) null else page - 1
                val nextKey = if (endOfPaginationReached) null else page + 1
                val keys = imageEntities.map {
                    RemoteKeys(id = it.id, prevKey = prevKey, nextKey = nextKey)
                }
                remoteKeysDao.insertAll(keys)
                imagesDao.insertAll(imageEntities)
            }
            return MediatorResult.Success(endOfPaginationReached = endOfPaginationReached)
        } catch (exception: IOException) {
            return MediatorResult.Error(exception)
        } catch (exception: HttpException) {
            return MediatorResult.Error(exception)
        }
    }

    private suspend fun getRemoteKeyForLastItem(state: PagingState<Int, PixabayImageEntity>): RemoteKeys? {
        return state.pages.lastOrNull { it.data.isNotEmpty() }?.data?.lastOrNull()
            ?.let { pixabayImage ->
                remoteKeysDao.remoteKeysImageId(pixabayImage.id)
            }
    }

    private suspend fun getRemoteKeyForFirstItem(state: PagingState<Int, PixabayImageEntity>): RemoteKeys? {
        return state.pages.firstOrNull { it.data.isNotEmpty() }?.data?.firstOrNull()
            ?.let { pixabayImage ->
                remoteKeysDao.remoteKeysImageId(pixabayImage.id)
            }
    }

    private suspend fun getRemoteKeyClosestToCurrentPosition(
        state: PagingState<Int, PixabayImageEntity>
    ): RemoteKeys? {
        // The paging library is trying to load data after the anchor position
        // Get the item closest to the anchor position
        return state.anchorPosition?.let { position ->
            state.closestItemToPosition(position)?.id?.let { imageId ->
                remoteKeysDao.remoteKeysImageId(imageId)
            }
        }
    }
}