package dev.adriankuta.pixabay.feature.search

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.statusBarsPadding
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.material3.ListItem
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.ui.Modifier
import androidx.compose.ui.layout.ContentScale
import androidx.compose.ui.platform.testTag
import androidx.compose.ui.res.stringResource
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.paging.LoadState
import androidx.paging.compose.LazyPagingItems
import androidx.paging.compose.collectAsLazyPagingItems
import coil.compose.AsyncImage
import dev.adriankuta.pixabay.core.ui.ErrorMessage
import dev.adriankuta.pixabay.core.ui.Loading
import dev.adriankuta.pixabay.core.ui.SearchField
import dev.adriankuta.pixabay.data.model.PixabayImage

@Composable
fun SearchRoute(
    onGoToItem: (Int) -> Unit,
    modifier: Modifier = Modifier,
    viewModel: SearchScreenViewModel = hiltViewModel()
) {
    val state by viewModel.state.collectAsState()
    val imagePagingItems = viewModel.pagingDataFlow.collectAsLazyPagingItems()
    SearchScreen(
        state = state,
        pagingData = imagePagingItems,
        onGoToItem = onGoToItem,
        onQueryChange = viewModel::onQueryChange,
        modifier = modifier
    )
}

@Composable
internal fun SearchScreen(
    state: SearchUiState,
    pagingData: LazyPagingItems<PixabayImage>,
    onGoToItem: (Int) -> Unit,
    onQueryChange: (String) -> Unit,
    modifier: Modifier = Modifier,
) {
    Scaffold(
        topBar = {
            SearchField(
                query = state.query,
                onQueryChange = onQueryChange,
                modifier = Modifier
                    .statusBarsPadding()
                    .fillMaxWidth()
                    .padding(horizontal = 16.dp),
                hint = stringResource(R.string.search)
            )
        }
    ) { paddingValues ->
        Content(
            modifier.padding(paddingValues),
            pagingData,
            onGoToItem
        )
    }
}

@Composable
internal fun Content(
    modifier: Modifier = Modifier,
    pagingData: LazyPagingItems<PixabayImage>,
    onGoToItem: (Int) -> Unit
) {

    LazyColumn(
        modifier = modifier.fillMaxSize(),
        contentPadding = PaddingValues(vertical = 8.dp)
    ) {
        items(pagingData.itemCount) { index ->
            val item = pagingData[index]!!
            ListItem(item, onGoToItem, index)
        }

        pagingData.apply {
            when {
                loadState.refresh is LoadState.Loading -> {
                    item { Loading() }
                }

                loadState.refresh is LoadState.Error -> {
                    val error = pagingData.loadState.refresh as LoadState.Error
                    item {
                        ErrorMessage(
                            modifier = Modifier
                                .padding(horizontal = 8.dp),
                            message = error.error.localizedMessage!!,
                            onClickRetry = { retry() })
                    }
                }

                loadState.append is LoadState.Loading -> {
                    item { Loading() }
                }

                loadState.append is LoadState.Error -> {
                    val error = pagingData.loadState.append as LoadState.Error
                    item {
                        ErrorMessage(
                            modifier = Modifier.padding(horizontal = 8.dp),
                            message = error.error.localizedMessage!!,
                            onClickRetry = { retry() })
                    }
                }
            }
        }
    }
}

@Composable
private fun ListItem(
    item: PixabayImage,
    onGoToItem: (Int) -> Unit,
    index: Int
) {
    ListItem(headlineContent = { Text(text = item.user) },
        supportingContent = { Text(text = item.tags) },
        leadingContent = {
            AsyncImage(
                model = item.previewURL,
                contentDescription = null,
                Modifier.fillMaxWidth(0.4f),
                contentScale = ContentScale.FillWidth,
            )
        },
        modifier = Modifier
            .clickable {
                onGoToItem(item.id)
            }
            .testTag("item_$index")
    )
}